package com.iamberry.wechat.handles.personality;

import com.google.gson.Gson;
import com.iamberry.wechat.core.entity.ResultMsg;
import com.iamberry.wechat.core.entity.WechatUtils;
import com.iamberry.wechat.core.entity.member.Member;
import com.iamberry.wechat.core.entity.personality.*;
import com.iamberry.wechat.face.Personality.PersonalitySubjectService;
import com.iamberry.wechat.face.home.HomeService;
import com.iamberry.wechat.face.member.MemberService;
import com.iamberry.wechat.tools.ResultInfo;
import com.iamberry.wechat.tools.WeixinUtil;
import com.iamberry.wechat.utils.ImageUtil;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map.Entry;
import java.util.*;

@Controller
@RequestMapping("/wechat/personality")
public class personalityHandler {	// 日志
    private  static org.slf4j.Logger logger = LoggerFactory.getLogger(WeixinUtil.class);

    @Autowired
    private PersonalitySubjectService personalitySubjectService;
    @Autowired
    private HomeService homeService;
    @Autowired
    private MemberService memberService;
    /**
     * 获取题目
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/listSubject")
    public ResultMsg listSubject(HttpServletRequest request) {
        ResultMsg msg = new ResultMsg();
        /*Member member = WechatUtils.getUserBySession(request);*/
        List<PersonalitySubject> listPersonalitySubject = personalitySubjectService.listSubject();
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("listPersonalitySubject", listPersonalitySubject);
        msg.setResultCode(ResultInfo.SUCCESSCODE);
        msg.setStatus(true);
        msg.setData(map);
        return msg;
    }
    /**
     * 获取题目答案
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getAnswer")
    public ResultMsg getAnswer(HttpServletRequest request,String subjectId) {
        ResultMsg msg = new ResultMsg();
        /*Member member = WechatUtils.getUserBySession(request);*/
        List<PersonalityAnswer> listAnswer = personalitySubjectService.getAnswer(Integer.valueOf(subjectId));
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("listAnswer", listAnswer);
        msg.setResultCode(ResultInfo.SUCCESSCODE);
        msg.setStatus(true);
        msg.setData(map);
        return msg;
    }
    /**
     * 获取推荐内容
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getRecommended")
    public ResultMsg getPersonalityRecommended(HttpServletRequest request,PersonalityRecommended personalityRecommended) {
        ResultMsg msg = new ResultMsg();
        PersonalityRecommended recommended = personalitySubjectService.getRecommended(personalityRecommended.getRecommendedTypeId());

        Map<String,Object> map = new HashMap<String,Object>();
        map.put("recommended", recommended);
        msg.setResultCode(ResultInfo.SUCCESSCODE);
        msg.setStatus(true);
        msg.setData(map);
        return msg;
    }
    /**
     * 计算测试结果
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/calculationResults")
    public ResultMsg calculationResults(HttpServletRequest request,String answers) throws IOException {
        ResultMsg msg = new ResultMsg();
        JSONObject jsonObject = null;
        String nickname =  null;
        String headimgurl = null;
        Member member = WechatUtils.getUserBySession(request);
        Member info = homeService.selectMemberInfoByOpenId(member.getUserOpenid());
        if(info == null || info.getUserNickname().equals("")){
            jsonObject = WeixinUtil.getUserInfo(member.getUserOpenid());
            logger.error("jsonObject22"+jsonObject);
            assert jsonObject != null;
            assert jsonObject.has("nickname");
            nickname = jsonObject.getString("nickname");
            headimgurl = jsonObject.getString("headimgurl");
        }else{
            nickname = info.getUserNickname();
        }

        try {
                String s = new String(Base64.getEncoder().encode(nickname.getBytes()), "UTF-8");
                new String(Base64.getDecoder().decode(s));
        } catch (UnsupportedEncodingException e) {
            nickname = "";
        }

        Member mem = new Member();
        mem.setUserHead(headimgurl);
        mem.setUserNickname(nickname);
        mem.setUserOpenid(member.getUserOpenid());

        Gson gson = new Gson();
        Map<String, Object> map = new HashMap<String, Object>();
        map = gson.fromJson(answers, map.getClass());

        //查询所有类型
        List<PersonalityType> listType = personalitySubjectService.getType();
        HashMap<Integer,Integer> answerScoremap = new HashMap<Integer,Integer>();
        for(PersonalityType personalityType : listType){
            answerScoremap.put(personalityType.getTypeId(),0);
        }
        for(int i = 0; i< map.size(); i++ ){
            String answerScore = (String)map.get(i+1+"");
            //查询所属类型
            PersonalitySubject subject = personalitySubjectService.getSubject(i+1);
            //计算各个类型所得分数
            answerScoremap.put(subject.getSubjectType(),answerScoremap.get(subject.getSubjectType()) + Integer.valueOf(answerScore));
        }
        //用中间值减去得分
        for(int i= 0;i<answerScoremap.size();i++){
            Integer typeScore = listType.get(i).getTypeScore();
            answerScoremap.put(i+1,typeScore - answerScoremap.get(i+1));
        }

        //计算最低分
        Object minKey = getMapMinOrMaxValueKey(answerScoremap, "max");
        PersonalityRecommended minRecommended = personalitySubjectService.getRecommended((Integer)minKey);
        //计算最高分
        Object maxKey = getMapMinOrMaxValueKey(answerScoremap, "min");
        PersonalityRecommended maxRecommended = personalitySubjectService.getRecommended((Integer)maxKey);

        String filePath = request.getServletContext().getRealPath("/common/images");
        File dir = new File(filePath + File.separator + "send");
        if (!dir.exists()) {
            dir.mkdir();
        }

        //图片添加昵称
        Font font = new Font("04b_08", Font.BOLD, 25);//字体
        String watermark= nickname + ":";//水印内容
        Color color=new Color(0,0,0,150);

        String sourceImg=filePath +minRecommended.getRecommendedLowUrl(); //源图片地址
        String targetImg=dir.getAbsolutePath() + File.separator+member.getUserOpenid()+"1.png".replaceAll("\\\\", "/"); //新存储的地址
        ImageUtil.addWatermark(sourceImg, targetImg,watermark, color,font);

        String sourceImg2=filePath +maxRecommended.getRecommendedHighUrl(); //源图片地址
        String targetImg2=dir.getAbsolutePath() + File.separator+member.getUserOpenid()+"2.png".replaceAll("\\\\", "/"); //新存储的地址
        ImageUtil.addWatermark(sourceImg2, targetImg2,watermark, color,font);

        String[] files1 = new String[2];
        files1[0] = (filePath + File.separator+"send"+File.separator+member.getUserOpenid()+"2.png").replaceAll("\\\\", "/");
        files1[1] = (filePath + File.separator+"db2.png").replaceAll("\\\\", "/");
        ImageUtil.mergeImage(files1,2,dir.getAbsolutePath() + File.separator+member.getUserOpenid()+"db.png");


        String[] files = new String[2];
        files[0] = (filePath + File.separator+"send"+File.separator+member.getUserOpenid()+"1.png").replaceAll("\\\\", "/");
        files[1] = (filePath + File.separator+"send"+File.separator+member.getUserOpenid()+"db.png").replaceAll("\\\\", "/");
        ImageUtil.mergeImage(files,2,dir.getAbsolutePath() + File.separator+member.getUserOpenid()+".png");

        //获取生成后的图片地址
        String url = ("https://w.iamberry.com/wechat_rg/common/images/send/"+member.getUserOpenid()+".png");

        //添加测试结果
        PersonalityResults personalityResults = new PersonalityResults();
        personalityResults.setResultsOpenid(member.getUserOpenid());
        personalityResults.setResultsUrl(url);
        personalitySubjectService.addResults(personalityResults);

        Map<String,Object> map2 = new HashMap<String,Object>();
        map2.put("url", url);
        msg.setResultCode(ResultInfo.SUCCESSCODE);
        msg.setStatus(true);
        msg.setData(map2);
        return msg;
    }

    public static Object getMapMinOrMaxValueKey(Map<Integer, Integer> map, String choose) {
        List<Entry<Integer,Integer>> list = new ArrayList(map.entrySet());
        Collections.sort(list, (o1, o2) -> (o1.getValue().intValue() - o2.getValue().intValue()));
        Object key = null;
        if (choose.equals("min")) {
            key = list.get(0).getKey();
        } else if (choose.equals("max")) {
            key = list.get(list.size() - 1).getKey();
        }
        return key;
    }

    /**
     * 判断是否已存在测试结果
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getResults")
    public ResultMsg getResults(HttpServletRequest request) {
        ResultMsg msg = new ResultMsg();
        Member member = WechatUtils.getUserBySession(request);
        PersonalityResults personalityResults = personalitySubjectService.getResults(member.getUserOpenid());
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("personalityResults", personalityResults);
        msg.setResultCode(ResultInfo.SUCCESSCODE);
        msg.setStatus(true);
        msg.setData(map);
        return msg;
    }
}
